/*
 * Copyright (C) 2009 Thomas Chou <thomas@wytron.com.tw>
 *
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, GOOD TITLE or
 * NON INFRINGEMENT.  See the GNU General Public License for more
 * details.
 */

#ifndef _ASM_NIOS2_LINKAGE_H
#define _ASM_NIOS2_LINKAGE_H

/* This file is required by include/linux/linkage.h */
#define __ALIGN .align 4
#define __ALIGN_STR ".align 4"

#endif
